from django.contrib.auth import get_user_model
from django.test import RequestFactory, TestCase
from django.urls import reverse
from django.views.generic.edit import UpdateView

from django.conf import settings

from time import sleep

from ..models import *
from ..utils import *
from ..wallet_utils import *
from ..registration_utils import *
from ..agent_utils import *


class AgentInteractionTests(TestCase):

    def create_user_and_org(self):
        # create, register and provision a user and org
        # create, register and provision a user
        email = random_alpha_string(10) + "@agent_utils.com"
        user_wallet_name = get_user_wallet_name(email)
        user = get_user_model().objects.create(
            email=email,
            first_name='Test',
            last_name='Registration',
        )
        user.save()
        raw_password = random_alpha_string(8)
        user_provision(user, raw_password)

        # now org
        org_name = 'Agent Utils ' + random_alpha_string(10)
        org = org_signup(user, raw_password, org_name)

        return (user, org, raw_password)

    def establish_agent_connection(self, org, user):
        # send connection request (org -> user)
        org_connection_1 = send_connection_invitation(org.wallet, user.email)
        sleep(1)

        # accept connection request (user -> org)
        user_connection = send_connection_confirmation(user.wallet, org_connection_1.id, org.org_name, org_connection_1.invitation)
        sleep(1)

        # update connection status (org)
        org_connection_2 = check_connection_status(org.wallet, org_connection_1)

        return (org_connection_2, user_connection)

    def delete_user_and_org_wallets(self, user, org, raw_password):
        # cleanup after ourselves
        org_wallet_name = org.wallet.wallet_name
        res = delete_wallet(org_wallet_name, raw_password)
        self.assertEqual(res, 0)
        user_wallet_name = user.wallet.wallet_name
        res = delete_wallet(user_wallet_name, raw_password)
        self.assertEqual(res, 0)

    def schema_and_cred_def_for_org(self, org):
        # create a "dummy" schema/cred-def that is unique to this org (matches the Alice/Faber demo schema)
        wallet = org.wallet
        wallet_name = org.wallet.wallet_name

        (schema_json, creddef_template) = create_schema_json('schema_' + wallet_name, random_schema_version(), [
            'name', 'date', 'degree', 'age',
            ])
        schema = create_schema(wallet, schema_json, creddef_template)
        cred_def = create_creddef(wallet, schema, 'creddef_' + wallet_name, creddef_template)

         # Proof of Age
        proof_request = create_proof_request('Proof of Age Test', 'Proof of Age Test',
            [{'name':'name', 'restrictions':[{'issuer_did': '$ISSUER_DID'}]},
             {'name':'comments'}],
            [{'name': 'age','p_type': '>=','p_value': '$VALUE', 'restrictions':[{'issuer_did': '$ISSUER_DID'}]}]
            )

        return (schema, cred_def, proof_request)


    def issue_credential_from_org_to_user(self, org, user, org_connection, user_connection, cred_def, schema_attrs, cred_name, cred_tag):
        # issue a credential based on the default schema/credential definition
        org_conversation_1 = send_credential_offer(org.wallet, org_connection,  
                                            cred_tag, schema_attrs, cred_def, 
                                            cred_name)
        sleep(2)

        i = 0
        while True:
            handled_count = handle_inbound_messages(user.wallet, user_connection)
            i = i + 1
            if handled_count > 0 or i > 3:
                break
            sleep(2)
        user_conversations = AgentConversation.objects.filter(connection__wallet=user.wallet, conversation_type="CredentialOffer", status='Pending').all()
        user_conversation_1 = user_conversations[0]

        # send credential request (user -> org)
        user_conversation_2 = send_credential_request(user.wallet, user_connection, user_conversation_1)
        sleep(2)

        # send credential (org -> user)
        i = 0
        message = org_conversation_1
        while True:
            message = poll_message_conversation(org.wallet, org_connection, message, initialize_vcx=True)
            i = i + 1
            if message.conversation_type == 'IssueCredential' or i > 3:
                break
            sleep(2)
        org_conversation_2 = message
        sleep(2)

        # accept credential and update status (user)
        i = 0
        message = user_conversation_2
        while True:
            message = poll_message_conversation(user.wallet, user_connection, message, initialize_vcx=True)
            i = i + 1
            if message.status == 'Accepted' or i > 3:
                break
            sleep(2)
        user_conversation_3 = message
        sleep(2)

        # update credential offer status (org)
        i = 0
        message = org_conversation_2
        while True:
            message = poll_message_conversation(org.wallet, org_connection, message, initialize_vcx=True)
            i = i + 1
            if message.status == 'Accepted' or i > 3:
                break
            sleep(2)
        org_conversation_3 = message

    def issue_credential_from_org_to_user_bg_tasks(self, org, user, org_connection, user_connection, cred_def, schema_attrs, cred_name, cred_tag):
        # issue a credential based on the default schema/credential definition
        org_conversation_1 = send_credential_offer(org.wallet, org_connection,  
                                            cred_tag, schema_attrs, cred_def, 
                                            cred_name)
        sleep(2)

        i = 0
        while True:
            handled_count = handle_inbound_messages(user.wallet, user_connection)
            i = i + 1
            if handled_count > 0 or i > 3:
                break
            sleep(2)
        user_conversations = AgentConversation.objects.filter(connection__wallet=user.wallet, conversation_type="CredentialOffer", status='Pending').all()
        user_conversation_1 = user_conversations[0]

        # send credential request (user -> org)
        user_conversation_2 = send_credential_request(user.wallet, user_connection, user_conversation_1)
        sleep(2)

        # send credential (org -> user)
        i = 0
        message = org_conversation_1
        while True:
            poll_count = poll_message_conversations(org.wallet, org_connection)
            i = i + 1
            if poll_count == 1 or i > 3:
                break
            sleep(2)
        org_conversation_2 = AgentConversation.objects.filter(connection__wallet=org.wallet, id=message.id).all()[0]
        if org_conversation_2.conversation_type != 'IssueCredential':
            raise Exception("Expected IssueCredential but got " + org_conversation_2.conversation_type)
        sleep(2)

        # accept credential and update status (user)
        i = 0
        message = user_conversation_2
        while True:
            poll_count = poll_message_conversations(user.wallet, user_connection)
            i = i + 1
            if poll_count == 1 or i > 3:
                break
            sleep(2)
        user_conversation_3 = AgentConversation.objects.filter(connection__wallet=user.wallet, id=message.id).all()[0]
        if user_conversation_3.status != 'Accepted':
            raise Exception('Expected Accepted but got' + user_conversation_3.status)
        sleep(2)

        # update credential offer status (org)
        i = 0
        message = org_conversation_2
        while True:
            poll_count = poll_message_conversations(org.wallet, org_connection)
            i = i + 1
            if poll_count == 1 or i > 3:
                break
            sleep(2)
        org_conversation_3 = AgentConversation.objects.filter(connection__wallet=org.wallet, id=message.id).all()[0]
        if org_conversation_3.status != 'Accepted':
            raise Exception('Expected Accepted but got' + org_conversation_3.status)


    def test_register_org_with_schema_and_cred_def(self):
        # try creating a schema and credential definition under the organization
        (user, org, raw_password) = self.create_user_and_org()
        (schema, cred_def, proof_request) = self.schema_and_cred_def_for_org(org)

        # fetch some stuff and validate some other stuff
        fetch_org = IndyOrganization.objects.filter(org_name=org.org_name).all()[0]
        self.assertEqual(len(fetch_org.wallet.indycreddef_set.all()), 1)
        fetch_creddef = fetch_org.wallet.indycreddef_set.all()[0]
        self.assertEqual(fetch_creddef.creddef_name, cred_def.creddef_name)

        # clean up after ourself
        self.delete_user_and_org_wallets(user, org, raw_password)


    def test_agent_connection(self):
        # establish a connection between two agents
        (user, org, raw_password) = self.create_user_and_org()
        (schema, cred_def, proof_request) = self.schema_and_cred_def_for_org(org)

        (org_connection, user_connection) = self.establish_agent_connection(org, user)
        self.assertEqual(org_connection.status, 'Active')

        # clean up after ourself
        self.delete_user_and_org_wallets(user, org, raw_password)


    def test_agent_credential_exchange(self):
        # exchange credentials between two agents
        (user, org, raw_password) = self.create_user_and_org()
        (schema, cred_def, proof_request) = self.schema_and_cred_def_for_org(org)

        # establish a connection
        (org_connection, user_connection) = self.establish_agent_connection(org, user)

        # issue credential offer (org -> user)
        schema_attrs = json.loads(cred_def.creddef_template)
        # data normally provided by the org data pipeline
        schema_attrs['name'] = 'Joe Smith'
        schema_attrs['date'] = '2018-01-01'
        schema_attrs['degree'] = 'B.A.Sc. Honours'
        schema_attrs['age'] = '25'
        org_conversation_1 = send_credential_offer(org.wallet, org_connection,  
                                            'Some Tag Value', schema_attrs, cred_def, 
                                            'Some Credential Name')
        sleep(2)

        # poll to receive credential offer
        user_conversations = AgentConversation.objects.filter(connection__wallet=user.wallet, conversation_type="CredentialOffer", status='Pending').all()
        self.assertEqual(len(user_conversations), 0)
        user_credentials = list_wallet_credentials(user.wallet)
        self.assertEqual(len(user_credentials), 0)

        i = 0
        while True:
            handled_count = handle_inbound_messages(user.wallet, user_connection)
            i = i + 1
            if handled_count > 0 or i > 3:
                break
            sleep(2)
        self.assertEqual(handled_count, 1)
        user_conversations = AgentConversation.objects.filter(connection__wallet=user.wallet, conversation_type="CredentialOffer", status='Pending').all()
        self.assertEqual(len(user_conversations), 1)
        user_conversation_1 = user_conversations[0]

        # send credential request (user -> org)
        user_conversation_2 = send_credential_request(user.wallet, user_connection, user_conversation_1)
        sleep(2)

        # send credential (org -> user)
        i = 0
        message = org_conversation_1
        while True:
            message = poll_message_conversation(org.wallet, org_connection, message, initialize_vcx=True)
            i = i + 1
            if message.conversation_type == 'IssueCredential' or i > 3:
                break
            sleep(2)
        self.assertEqual(message.conversation_type, 'IssueCredential')
        org_conversation_2 = message
        sleep(2)

        # accept credential and update status (user)
        i = 0
        message = user_conversation_2
        while True:
            message = poll_message_conversation(user.wallet, user_connection, message, initialize_vcx=True)
            i = i + 1
            if message.status == 'Accepted' or i > 3:
                break
            sleep(2)
        self.assertEqual(message.status, 'Accepted')
        user_conversation_3 = message
        sleep(2)

        # update credential offer status (org)
        i = 0
        message = org_conversation_2
        while True:
            message = poll_message_conversation(org.wallet, org_connection, message, initialize_vcx=True)
            i = i + 1
            if message.status == 'Accepted' or i > 3:
                break
            sleep(2)
        self.assertEqual(message.status, 'Accepted')
        org_conversation_3 = message

        # verify credential is in user wallet
        user_credentials = list_wallet_credentials(user.wallet)
        self.assertEqual(len(user_credentials), 1)

        # clean up after ourself
        self.delete_user_and_org_wallets(user, org, raw_password)


    def test_agent_credential_exchange_bgtask(self):
        # request and deliver a proof between two agents
        (user, org, raw_password) = self.create_user_and_org()
        (schema, cred_def, proof_request) = self.schema_and_cred_def_for_org(org)

        # establish a connection
        (org_connection, user_connection) = self.establish_agent_connection(org, user)

        # make up a credential
        schema_attrs = json.loads(cred_def.creddef_template)
        schema_attrs['name'] = 'Joe Smith'
        schema_attrs['date'] = '2018-01-01'
        schema_attrs['degree'] = 'B.A.Sc. Honours'
        schema_attrs['age'] = '25'
        cred_name = 'Cred4Proof Credential Name'
        cred_tag = 'Cred4Proof Tag Value'

        # issue credential (org -> user)
        self.issue_credential_from_org_to_user_bg_tasks(org, user, org_connection, user_connection, cred_def, schema_attrs, cred_name, cred_tag)

        # verify credential is in user wallet
        user_credentials = list_wallet_credentials(user.wallet)
        self.assertEqual(len(user_credentials), 1)

        # clean up after ourself
        self.delete_user_and_org_wallets(user, org, raw_password)


    def test_agent_proof_exchange(self):
        # request and deliver a proof between two agents
        (user, org, raw_password) = self.create_user_and_org()
        (schema, cred_def, proof_request) = self.schema_and_cred_def_for_org(org)

        # establish a connection
        (org_connection, user_connection) = self.establish_agent_connection(org, user)

        # make up a credential
        schema_attrs = json.loads(cred_def.creddef_template)
        schema_attrs['name'] = 'Joe Smith'
        schema_attrs['date'] = '2018-01-01'
        schema_attrs['degree'] = 'B.A.Sc. Honours'
        schema_attrs['age'] = '25'
        cred_name = 'Cred4Proof Credential Name'
        cred_tag = 'Cred4Proof Tag Value'

        # issue credential (org -> user)
        self.issue_credential_from_org_to_user(org, user, org_connection, user_connection, cred_def, schema_attrs, cred_name, cred_tag)

        # verify credential is in user wallet
        user_credentials = list_wallet_credentials(user.wallet)
        self.assertEqual(len(user_credentials), 1)

        # construct the proof request to send to the user (to whom we have just issued a credential)
        proof_req_attrs = proof_request.proof_req_attrs
        proof_req_predicates = proof_request.proof_req_predicates
        org_connection_data = json.loads(org_connection.connection_data)
        issuer_did = org_connection_data['data']['public_did']
        proof_req_attrs = proof_req_attrs.replace('$ISSUER_DID', issuer_did)
        proof_req_predicates = proof_req_predicates.replace('$ISSUER_DID', issuer_did)
        proof_req_predicates = proof_req_predicates.replace('"$VALUE"', '21')
        proof_req_attrs = json.loads(proof_req_attrs)
        proof_req_predicates = json.loads(proof_req_predicates)
        proof_name = "My Cool Proof"
        proof_uuid = "Some Uuid Value"

        # issue proof request (org -> user)
        org_conversation_1 = send_proof_request(org.wallet, org_connection, proof_uuid, proof_name, proof_req_attrs, proof_req_predicates)
        sleep(2)

        # poll to receive proof request
        user_conversations = AgentConversation.objects.filter(connection__wallet=user.wallet, conversation_type="ProofRequest", status='Pending').all()
        self.assertEqual(len(user_conversations), 0)

        i = 0
        while True:
            handled_count = handle_inbound_messages(user.wallet, user_connection)
            i = i + 1
            if handled_count > 0 or i > 3:
                break
            sleep(2)
        self.assertEqual(handled_count, 1)
        user_conversations = AgentConversation.objects.filter(connection__wallet=user.wallet, conversation_type="ProofRequest", status='Pending').all()
        self.assertEqual(len(user_conversations), 1)
        user_conversation_1 = user_conversations[0]

        # select credential(s) for proof (user)
        claim_data = get_claims_for_proof_request(user.wallet, user_connection, user_conversation_1)
        credential_attrs = {}
        for attr in claim_data['attrs']:
            # build array of credential id's (from wallet)
            claims = claim_data['attrs'][attr]
            if 0 < len(claims):
                self.assertEqual(len(claims), 1)
                credential_attrs[attr] = {'referent': claims[0]['cred_info']['referent']}
            else:
                # if no claim available, make up a self-attested value
                self.assertEqual(attr, 'comments')
                credential_attrs[attr] = {'value': 'user-supplied value'}

        # construct proof and send (user -> org)
        user_conversation_2 = send_claims_for_proof_request(user.wallet, user_connection, user_conversation_1, credential_attrs)

        # accept and validate proof (org)
        i = 0
        message = org_conversation_1
        while True:
            message = poll_message_conversation(org.wallet, org_connection, message, initialize_vcx=True)
            i = i + 1
            if message.status == 'Accepted' or i > 3:
                break
            sleep(2)
        self.assertEqual(message.status, 'Accepted')
        self.assertEqual(message.proof_state, 'Verified')
        org_conversation_2 = message

        # TODO verify some stuff ...

        # clean up after ourself
        self.delete_user_and_org_wallets(user, org, raw_password)


    def test_agent_proof_exchange_2_creds(self):
        # request and deliver a proof between two agents
        (user, org, raw_password) = self.create_user_and_org()
        (schema, cred_def, proof_request) = self.schema_and_cred_def_for_org(org)

        # establish a connection
        (org_connection, user_connection) = self.establish_agent_connection(org, user)

        # make up a credential
        schema_attrs = json.loads(cred_def.creddef_template)
        schema_attrs['name'] = 'Joe Smith'
        schema_attrs['date'] = '2018-01-01'
        schema_attrs['degree'] = 'B.A.Sc. Honours'
        schema_attrs['age'] = '25'
        cred_name = 'Cred4Proof Credential Name'
        cred_tag = 'Cred4Proof Tag Value'

        # issue credential (org -> user)
        self.issue_credential_from_org_to_user(org, user, org_connection, user_connection, cred_def, schema_attrs, cred_name, cred_tag)

        # verify credential is in user wallet
        user_credentials = list_wallet_credentials(user.wallet)
        self.assertEqual(len(user_credentials), 1)

        # make up a credential
        schema_attrs = json.loads(cred_def.creddef_template)
        schema_attrs['name'] = 'Joe Smith 2'
        schema_attrs['date'] = '2018-01-02'
        schema_attrs['degree'] = 'M.Sc. Honours'
        schema_attrs['age'] = '28'
        cred_name = 'Cred4Proof Credential Name 2'
        cred_tag = 'Cred4Proof Tag Value 2'

        # issue credential (org -> user)
        self.issue_credential_from_org_to_user(org, user, org_connection, user_connection, cred_def, schema_attrs, cred_name, cred_tag)

        # verify credential is in user wallet
        user_credentials = list_wallet_credentials(user.wallet)
        self.assertEqual(len(user_credentials), 2)

        # construct the proof request to send to the user (to whom we have just issued a credential)
        proof_req_attrs = proof_request.proof_req_attrs
        proof_req_predicates = proof_request.proof_req_predicates
        org_connection_data = json.loads(org_connection.connection_data)
        issuer_did = org_connection_data['data']['public_did']
        proof_req_attrs = proof_req_attrs.replace('$ISSUER_DID', issuer_did)
        proof_req_predicates = proof_req_predicates.replace('$ISSUER_DID', issuer_did)
        proof_req_predicates = proof_req_predicates.replace('"$VALUE"', '21')
        proof_req_attrs = json.loads(proof_req_attrs)
        proof_req_predicates = json.loads(proof_req_predicates)
        proof_name = "My Cool Proof"
        proof_uuid = "Some Uuid Value"

        # issue proof request (org -> user)
        org_conversation_1 = send_proof_request(org.wallet, org_connection, proof_uuid, proof_name, proof_req_attrs, proof_req_predicates)
        sleep(2)

        # poll to receive proof request
        user_conversations = AgentConversation.objects.filter(connection__wallet=user.wallet, conversation_type="ProofRequest", status='Pending').all()
        self.assertEqual(len(user_conversations), 0)

        i = 0
        while True:
            handled_count = handle_inbound_messages(user.wallet, user_connection)
            i = i + 1
            if handled_count > 0 or i > 3:
                break
            sleep(2)
        self.assertEqual(handled_count, 1)
        user_conversations = AgentConversation.objects.filter(connection__wallet=user.wallet, conversation_type="ProofRequest", status='Pending').all()
        self.assertEqual(len(user_conversations), 1)
        user_conversation_1 = user_conversations[0]

        # select credential(s) for proof (user) - there are 2 creds now
        claim_data = get_claims_for_proof_request(user.wallet, user_connection, user_conversation_1)
        for attr in claim_data['attrs']:
            # build array of credential id's (from wallet)
            claims = claim_data['attrs'][attr]
            if 0 < len(claims):
                self.assertEqual(len(claims), 2)
            else:
                self.assertEqual(attr, 'comments')

        # try again with additional_filters
        claim_data = get_claims_for_proof_request(user.wallet, user_connection, user_conversation_1, additional_filters={'degree':'M.Sc. Honours'})
        credential_attrs = {}
        for attr in claim_data['attrs']:
            # build array of credential id's (from wallet)
            claims = claim_data['attrs'][attr]
            if 0 < len(claims):
                self.assertEqual(len(claims), 1)
                credential_attrs[attr] = {'referent': claims[0]['cred_info']['referent']}
            else:
                # if no claim available, make up a self-attested value
                self.assertEqual(attr, 'comments')
                credential_attrs[attr] = {'value': 'user-supplied value'}

        # construct proof and send (user -> org)
        user_conversation_2 = send_claims_for_proof_request(user.wallet, user_connection, user_conversation_1, credential_attrs)

        # accept and validate proof (org)
        i = 0
        message = org_conversation_1
        while True:
            message = poll_message_conversation(org.wallet, org_connection, message, initialize_vcx=True)
            i = i + 1
            if message.status == 'Accepted' or i > 3:
                break
            sleep(2)
        self.assertEqual(message.status, 'Accepted')
        self.assertEqual(message.proof_state, 'Verified')
        org_conversation_2 = message

        # TODO verify some stuff ...

        # clean up after ourself
        self.delete_user_and_org_wallets(user, org, raw_password)




# Generated by Django 2.1.7 on 2019-03-28 02:48

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('indy_community', '0003_auto_20190327_1332'),
    ]

    operations = [
        migrations.AlterField(
            model_name='indyorganization',
            name='wallet',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='wallet_org', to='indy_community.IndyWallet', to_field='wallet_name'),
        ),
        migrations.AlterField(
            model_name='indyuser',
            name='wallet',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='wallet_user', to='indy_community.IndyWallet', to_field='wallet_name'),
        ),
    ]



from django.contrib.auth import get_user_model
from django.test import RequestFactory, TestCase
from django.urls import reverse
from django.views.generic.edit import UpdateView

import json
import time

from indy import anoncreds, crypto, did, ledger, pool, wallet
from indy.error import ErrorCode, IndyError

from ..ledger_utils import *
from ..wallet_utils import *
from ..indy_utils import *
from ..utils import *


LEDGER_SEED = '000000000000000000000000Trustee1'

ANON_DID    = "JGm22KSeapYEa8syXsE41a"
FABER_DID   = "U4mQRtGc7nURTidnkGGZTG"
ACME_DID    = "AhDeQ2udDSAzQzejYoFvc7"
THRIFT_DID  = "S88wA4cvCF8qPBtNndWaes"
BC_PROV_DID = "KtCGSAfKGES32W9wAeLUDA"

SCHEMA_ID   = "JGm22KSeapYEa8syXsE41a:2:Drivers License:27.34.12"
CRED_DEF_ID = "KtCGSAfKGES32W9wAeLUDA:3:CL:15:tag1"


class IndyLedgerTests(TestCase):
    """
    Tests for Indy ledger utility functions
    """

    def test_read_did_info_from_ledger(self):
        (nym_response, txn_response) = get_did_info(ANON_DID)
        #print("nym_response", nym_response)
        #print("txn_response", txn_response)

    def test_read_did_attrib_from_ledger(self):
        attrib_response = get_did_attrib(ANON_DID, "endpoint")
        #print("attrib_response(endpoint)", json.loads(attrib_response))

        attrib_response = get_did_attrib(ANON_DID, "alias")
        #print("attrib_response(alias)", json.loads(attrib_response))

    def test_write_did_attrib(self):
        raw = "{\"endpoint\":{\"ha\":\"127.0.0.1:5555\"}}"

        org_name = 'Some ' + random_alpha_string(10) + ' Inc'
        raw_password = 'pass1234'
        org_wallet_name = get_org_wallet_name(org_name)
        alias = org_name
        role = "TRUST_ANCHOR"

        # assume that if we run with no exceptions, we are successful
        res = create_wallet(org_wallet_name, raw_password)
        self.assertEqual(res, 0)
        wallet_handle = open_wallet(org_wallet_name, raw_password)

        # create a did and write to wallet
        ledger_did_info = json.dumps({'seed': LEDGER_SEED})
        did_info = json.dumps({'seed': calc_wallet_seed(org_name)})
        ledger_did, ledger_verkey = run_coroutine_with_args(did.create_and_store_my_did, wallet_handle, ledger_did_info)
        the_did, verkey = run_coroutine_with_args(did.create_and_store_my_did, wallet_handle, did_info)
        #print("did =", the_did)
        write_new_did(wallet_handle, ledger_did, the_did, verkey, alias, role)
        time.sleep(5)
        (nym_response, txn_response) = get_did_info(the_did)
        #print("nym_response:", nym_response)
        #print("txn_response:", txn_response)

        # now try to write a did attribute to the ledger
        write_did_attrib(wallet_handle, ledger_did, the_did, raw)

        # read it and see what we get
        attrib_response = get_did_attrib(the_did, "endpoint")
        #print("attrib_response(endpoint):", attrib_response)

        # close and delete wallet
        res = close_wallet(wallet_handle)
        self.assertEqual(res, 0)

        # cleanup after ourselves
        res = delete_wallet(org_wallet_name, raw_password)
        self.assertEqual(res, 0)

    def test_read_schema(self):
        schema_response = get_schema_info(ANON_DID, SCHEMA_ID)
        #print("schema", schema_response)

    def test_read_cred_def(self):
        cred_def_response = get_cred_def_info(ANON_DID, CRED_DEF_ID)
        #print("cred_def", cred_def_response)





from django.contrib.auth import get_user_model
from django.test import RequestFactory, TestCase
from django.urls import reverse
from django.views.generic.edit import UpdateView

from ..models import *
from ..wallet_utils import *


class WalletDBTests(TestCase):
    """
    Tests for Indy wallet model class
    """

    def test_user_wallet_db_lifecycle(self):
        # create a wallet
        user_name = 'user@mail.com'
        raw_password = 'pass1234'
        user_wallet_name = get_user_wallet_name(user_name)

        # assume that if we run with no exceptions, we are successful
        res = create_wallet(user_wallet_name, raw_password)
        self.assertEqual(res, 0)
        wallet_handle = open_wallet(user_wallet_name, raw_password)
        res = close_wallet(wallet_handle)
        self.assertEqual(res, 0)

        # cleanup after ourselves
        res = delete_wallet(user_wallet_name, raw_password)
        self.assertEqual(res, 0)


    def test_org_wallet_db_lifecycle(self):
        # create a wallet
        org_name = 'Some Company Inc'
        raw_password = 'pass1234'
        org_wallet_name = get_org_wallet_name(org_name)

        # assume that if we run with no exceptions, we are successful
        res = create_wallet(org_wallet_name, raw_password)
        self.assertEqual(res, 0)
        wallet_handle = open_wallet(org_wallet_name, raw_password)
        res = close_wallet(wallet_handle)
        self.assertEqual(res, 0)

        # cleanup after ourselves
        res = delete_wallet(org_wallet_name, raw_password)
        self.assertEqual(res, 0)





from django.contrib.auth.signals import user_logged_in, user_logged_out
from django.contrib.auth import get_user_model
from django.conf import settings

import json

from .models import *
from .tasks import *
from .wallet_utils import *


USER_ROLE = getattr(settings, "DEFAULT_USER_ROLE", 'User')
ORG_ROLE = getattr(settings, "DEFAULT_ORG_ROLE", 'Admin')

def url_indy_profile(role):
    if role == ORG_ROLE:
        return 'indy/base_organization_profile.html'
    else:
        return 'indy/base_individual_profile.html'

def is_organization_login(user, path):
    org_url = getattr(settings, 'ORG_NAMESPACE', None)
    if org_url:
        if org_url in path:
            return True
        return False
    if user.has_role(ORG_ROLE):
        print("User has org role", ORG_ROLE)
        return True
    return False

def user_wallet_logged_in_handler(request, user, wallet_name):
    print("Login wallet, {} {} {}".format(user.email, request.session.session_key, wallet_name))
    (session, session_created) = IndySession.objects.get_or_create(user=user, session_id=request.session.session_key)
    session.wallet_name = wallet_name
    session.save()

def user_wallet_logged_out_handler(request, user):
    print("Logout wallet, {} {}".format(user.email, request.session.session_key))
    session = IndySession.objects.get(user=user, session_id=request.session.session_key)
    session.wallet_name = None
    session.save()

def user_logged_in_handler(sender, user, request, **kwargs):
    if 'wallet_name' in request.session:
        wallet_name = request.session['wallet_name']
    else:
        wallet_name = None
    print("Login user {} {} {}".format(user.email, request.session.session_key, wallet_name))
    (session, session_created) = IndySession.objects.get_or_create(user=user, session_id=request.session.session_key, wallet_name=wallet_name)
    agent_background_task("Started by user login", user.id, request.session.session_key, repeat=AGENT_POLL_INTERVAL)


def user_logged_out_handler(sender, user, request, **kwargs):
    print("Logout user {} {}".format(user.email, request.session.session_key))
    IndySession.objects.get(user=user, session_id=request.session.session_key).delete()


def handle_wallet_login_internal(request, user, wallet_name, raw_password):
    # get user or org associated with this wallet
    related_user = get_user_model().objects.filter(wallet__wallet_name=wallet_name).all()
    related_org = IndyOrganization.objects.filter(wallet__wallet_name=wallet_name).all()
    if len(related_user) == 0 and len(related_org) == 0:
        raise Exception('Error wallet with no owner {}'.format(wallet_name))

    # now try to open the wallet - will throw an exception if it fails
    wallet_handle = open_wallet(wallet_name, raw_password)
    close_wallet(wallet_handle)

    if len(related_user) > 0:
        request.session['wallet_type'] = 'user'
        request.session['wallet_owner'] = related_user[0].email
    elif len(related_org) > 0:
        request.session['wallet_type'] = 'org'
        request.session['wallet_owner'] = related_org[0].org_name
    request.session['wallet_name'] = wallet_name
    request.session['wallet_password'] = raw_password

    user_wallet_logged_in_handler(request, user, wallet_name)


def handle_wallet_logout_internal(request):
    # clear wallet-related session variables
    if 'wallet_type' in request.session:
        del request.session['wallet_type']
    if 'wallet_name' in request.session:
        del request.session['wallet_name']
    if 'wallet_password' in request.session:
        del request.session['wallet_password']
    if 'wallet_owner' in request.session:
        del request.session['wallet_owner']


def init_user_session(sender, user, request, **kwargs):
    target = request.POST.get('next', '/profile/')
    if is_organization_login(user, target):
        request.session['ACTIVE_ROLE'] = ORG_ROLE
        orgs = IndyOrgRelationship.objects.filter(user=user).all()
        if 0 < len(orgs):
            sel_org = orgs[0].org
            request.session['ACTIVE_ORG'] = str(sel_org.id)

            # login as org wallet
            if sel_org.wallet is not None:
                sel_wallet = sel_org.wallet
                config = json.loads(sel_wallet.wallet_config)
                handle_wallet_login_internal(request, user, config['wallet_name'], config['wallet_key'])
    else:
        if user.has_role(USER_ROLE):
            request.session['ACTIVE_ROLE'] = USER_ROLE
        else:
            # TODO for now just set a dummy default - logged in user with no role assigned
            request.session['ACTIVE_ROLE'] = USER_ROLE

        # try to login as user wallet
        if user.wallet is not None:
            sel_wallet = user.wallet
            config = json.loads(sel_wallet.wallet_config)
            handle_wallet_login_internal(request, user, config['wallet_name'], config['wallet_key'])

    role = request.session['ACTIVE_ROLE']
    request.session['INDY_PROFILE'] = url_indy_profile(role)

    # setup background "virtual agent"
    user_logged_in_handler(sender, user, request, **kwargs)


def clear_user_session(sender, user, request, **kwargs):
    # setup background "virtual agent"
    user_logged_out_handler(sender, user, request, **kwargs)

    if 'ACTIVE_ROLE' in request.session:
        del request.session['ACTIVE_ROLE']
    if 'ACTIVE_ORG' in request.session:
        del request.session['ACTIVE_ORG']
    request.session['INDY_PROFILE'] = ''


user_logged_in.connect(init_user_session)

user_logged_out.connect(clear_user_session)





# Generated by Django 2.1.7 on 2019-04-30 03:31

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('indy_community', '0008_indyorganization_ico_url'),
    ]

    operations = [
        migrations.AddField(
            model_name='indyorganization',
            name='managed_wallet',
            field=models.BooleanField(default=True),
        ),
        migrations.AddField(
            model_name='indyuser',
            name='managed_wallet',
            field=models.BooleanField(default=True),
        ),
    ]



from django.contrib.auth import get_user_model
from django.test import RequestFactory, TestCase
from django.urls import reverse
from django.views.generic.edit import UpdateView

from indy_community.models import *
from indy_community.wallet_utils import *
from indy_community.utils import *

from ..indy import *
from ..record import *


class WalletStorageTests(TestCase):
    """
    Tests for Indy wallet model class
    """

    def test_wallet_storage_lifecycle(self):
        # create a wallet
        user_name = random_alpha_string(10) + '@mail.com'
        raw_password = 'pass1234'
        user_wallet_name = get_user_wallet_name(user_name)

        # assume that if we run with no exceptions, we are successful
        res = create_wallet(user_wallet_name, raw_password)
        self.assertEqual(res, 0)
        wallet_handle = open_wallet(user_wallet_name, raw_password)

        # open storage and do some stuff
        storage = IndyStorage(wallet_handle=wallet_handle)

        # store a record in non-secrets storage and then retrieve it
        test_record = StorageRecord(type="tester", value='{"Some": "Value", "Probably": "a", "Json": "Structure")', tags={"val1": "Value1", "val2": "Value2"})
        run_coroutine_with_args(storage.add_record, test_record)

        fetch_record = run_coroutine_with_kwargs(storage.get_record, record_type=test_record.type, record_id=test_record.id)

        run_coroutine_with_kwargs(storage.delete_record, record=test_record)

        res = close_wallet(wallet_handle)
        self.assertEqual(res, 0)

        # cleanup after ourselves
        res = delete_wallet(user_wallet_name, raw_password)
        self.assertEqual(res, 0)




import os
from setuptools import find_packages, setup

with open(os.path.join(os.path.dirname(__file__), 'README.rst')) as readme:
    README = readme.read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='django-indy-community',
    version='0.1.14',
    packages=find_packages(),
    include_package_data=True,
    license='996.ICU License',  # see https://github.com/996icu/996.ICU
    description='A simple Django package to build web-based Indy agent applications.',
    long_description=README,
    url='https://github.com/AnonSolutions/django-indy-community',
    author='Ian Costanzo',
    author_email='ian@anon-solutions.ca',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: Other/Proprietary License',  
        'Operating System :: Unix',
        'Operating System :: MacOS',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Topic :: Internet :: WWW/HTTP',
    ],
)



"""Basic in-memory storage implementation (non-wallet)."""

from collections import OrderedDict
from typing import Mapping, Sequence

from .base import BaseStorage, BaseStorageRecordSearch
from .error import (
    StorageError,
    StorageDuplicateError,
    StorageNotFoundError,
    StorageSearchError,
)
from .record import StorageRecord
#from ..wallet.base import BaseWallet


class BasicStorage(BaseStorage):
    """Basic in-memory storage class."""

    def __init__(self, _wallet_handle = None):
        """
        Initialize a `BasicStorage` instance.

        Args:
            _wallet_handle: The wallet implementation to use

        """
        self._records = OrderedDict()

    async def add_record(self, record: StorageRecord):
        """
        Add a new record to the store.

        Args:
            record: `StorageRecord` to be stored

        Raises:
            StorageError: If no record is provided
            StorageError: If the record has no ID

        """
        if not record:
            raise StorageError("No record provided")
        if not record.id:
            raise StorageError("Record has no ID")
        if record.id in self._records:
            raise StorageDuplicateError("Duplicate record")
        self._records[record.id] = record

    async def get_record(self, record_type: str, record_id: str) -> StorageRecord:
        """
        Fetch a record from the store by type and ID.

        Args:
            record_type: The record type
            record_id: The record id

        Returns:
            A `StorageRecord` instance

        Raises:
            StorageNotFoundError: If the record is not found

        """
        row = self._records.get(record_id)
        if row and row.type == record_type:
            return row
        if not row:
            raise StorageNotFoundError("Record not found: {}".format(record_id))

    async def update_record_value(self, record: StorageRecord, value: str):
        """
        Update an existing stored record's value.

        Args:
            record: `StorageRecord` to update
            value: The new value

        Raises:
            StorageNotFoundError: If record not found

        """
        oldrec = self._records.get(record.id)
        if not oldrec:
            raise StorageNotFoundError("Record not found: {}".format(record.id))
        self._records[record.id] = oldrec._replace(value=value)

    async def update_record_tags(self, record: StorageRecord, tags: Mapping):
        """
        Update an existing stored record's tags.

        Args:
            record: `StorageRecord` to update
            tags: New tags

        Raises:
            StorageNotFoundError: If record not found

        """
        oldrec = self._records.get(record.id)
        if not oldrec:
            raise StorageNotFoundError("Record not found: {}".format(record.id))
        self._records[record.id] = oldrec._replace(tags=dict(tags or {}))

    async def delete_record_tags(
        self, record: StorageRecord, tags: (Sequence, Mapping)
    ):
        """
        Update an existing stored record's tags.

        Args:
            record: `StorageRecord` to delete
            tags: Tags

        Raises:
            StorageNotFoundError: If record not found

        """
        oldrec = self._records.get(record.id)
        if not oldrec:
            raise StorageNotFoundError("Record not found: {}".format(record.id))
        newtags = dict(oldrec.tags or {})
        if tags:
            for tag in tags:
                if tag in newtags:
                    del newtags[tag]
        self._records[record.id] = oldrec._replace(tags=newtags)

    async def delete_record(self, record: StorageRecord):
        """
        Delete a record.

        Args:
            record: `StorageRecord` to delete

        Raises:
            StorageNotFoundError: If record not found

        """
        if record.id not in self._records:
            raise StorageNotFoundError("Record not found: {}".format(record.id))
        del self._records[record.id]

    def search_records(
        self, type_filter: str, tag_query: Mapping = None, page_size: int = None
    ) -> "BasicStorageRecordSearch":
        """
        Search stored records.

        Args:
            type_filter: Filter string
            tag_query: Tags to query
            page_size: Page size

        Returns:
            An instance of `BaseStorageRecordSearch`

        """
        return BasicStorageRecordSearch(self, type_filter, tag_query, page_size)


def basic_tag_query_match(tags: dict, tag_query: dict) -> bool:
    """Match simple tag filters (string values).

    TODO: implement more complex queries like $or
    """
    result = True
    if not tags:
        tags = {}
    if tag_query:
        for k, v in tag_query.items():
            if isinstance(v, str):  # not handling complex queries
                if tags.get(k) != v:
                    result = False
                    break
    return result


class BasicStorageRecordSearch(BaseStorageRecordSearch):
    """Represent an active stored records search."""

    def __init__(
        self,
        store: BasicStorage,
        type_filter: str,
        tag_query: Mapping,
        page_size: int = None,
    ):
        """
        Initialize a `BasicStorageRecordSearch` instance.

        Args:
            store: `BaseStorage` to search
            type_filter: Filter string
            tag_query: Tags to search
            page_size: Size of page to return

        """
        super(BasicStorageRecordSearch, self).__init__(
            store, type_filter, tag_query, page_size
        )
        self._cache = None
        self._iter = None

    @property
    def opened(self) -> bool:
        """
        Accessor for open state.

        Returns:
            True if opened, else False

        """
        return self._cache is not None

    async def fetch(self, max_count: int) -> Sequence[StorageRecord]:
        """
        Fetch the next list of results from the store.

        Args:
            max_count: Max number of records to return

        Returns:
            A list of `StorageRecord`s

        Raises:
            StorageSearchError: If the search query has not been opened

        """
        if not self.opened:
            raise StorageSearchError("Search query has not been opened")
        ret = []
        check_type = self.type_filter
        i = max_count
        while i > 0:
            try:
                id = next(self._iter)
            except StopIteration:
                break
            record = self._cache[id]
            if record.type == check_type and basic_tag_query_match(
                record.tags, self.tag_query
            ):
                ret.append(record)
                i -= 1
        return ret

    async def open(self):
        """Start the search query."""
        self._cache = self._store._records.copy()
        self._iter = iter(self._cache)

    async def close(self):
        """Dispose of the search query."""
        self._cache = None



import asyncio
import aiohttp
import json
from os import environ
import random

from django.conf import settings

from indy import anoncreds, crypto, did, ledger, pool, wallet
from indy.error import ErrorCode, IndyError
from indy.anoncreds import prover_search_credentials, prover_fetch_credentials, prover_close_credentials_search

from .models import *
from .utils import *


######################################################################
# basic wallet management utilities
######################################################################
def get_user_wallet_name(username):
    """
    Determine wallet name based on a user name (email).
    """

    wallet_name = username.replace("@", "_")
    wallet_name = wallet_name.replace(".", "_")
    return 'i_{}'.format(wallet_name).lower()


def get_org_wallet_name(orgname):
    """
    Determine wallet name based on an organization name.
    """

    wallet_name = orgname.replace("@", "_")
    wallet_name = wallet_name.replace(".", "_")
    wallet_name = wallet_name.replace(" ", "_")
    return 'o_{}'.format(wallet_name).lower()


def create_wallet(wallet_name, raw_password):
    """
    Create an Indy wallet (postgres).
    """

    wallet_config_json = wallet_config(wallet_name)
    wallet_credentials_json = wallet_credentials(raw_password)
    try:
        run_coroutine_with_args(wallet.create_wallet, wallet_config_json, wallet_credentials_json)
    except IndyError as ex:
        if ex.error_code == ErrorCode.WalletAlreadyExistsError:
            return 0
        return error_code
    return 0

def delete_wallet(wallet_name, raw_password):
    """
    Delete an Indy wallet (postgres).
    """

    wallet_config_json = wallet_config(wallet_name)
    wallet_credentials_json = wallet_credentials(raw_password)
    try:
        run_coroutine_with_args(wallet.delete_wallet, wallet_config_json, wallet_credentials_json)
    except IndyError as ex:
        return error_code
    return 0


def open_wallet(wallet_name, raw_password):
    """
    Open an Indy wallet (postgres).
    """

    wallet_config_json = wallet_config(wallet_name)
    wallet_credentials_json = wallet_credentials(raw_password)
    try:
        wallet_handle = run_coroutine_with_args(wallet.open_wallet, wallet_config_json, wallet_credentials_json)
        return wallet_handle
    except IndyError as ex:
        return error_code


def close_wallet(wallet_handle):
    """
    Close an Indy wallet (postgres).
    """

    try:
        run_coroutine_with_args(wallet.close_wallet, wallet_handle)
        return 0
    except IndyError as ex:
        return error_code


def wallet_config(wallet_name):
    """
    Build a wallet configuration dictionary (postgres specific).
    """

    storage_config = settings.INDY_CONFIG['storage_config']
    wallet_config = settings.INDY_CONFIG['wallet_config']
    wallet_config['id'] = wallet_name
    wallet_config['storage_config'] = storage_config
    wallet_config_json = json.dumps(wallet_config)
    return wallet_config_json


def wallet_credentials(raw_password):
    """
    Build wallet credentials dictionary (postgres specific).
    """

    storage_credentials = settings.INDY_CONFIG['storage_credentials']
    wallet_credentials = settings.INDY_CONFIG['wallet_credentials']
    wallet_credentials['key'] = raw_password
    wallet_credentials['storage_credentials'] = storage_credentials
    wallet_credentials_json = json.dumps(wallet_credentials)
    return wallet_credentials_json


def list_wallet_credentials(wallet):
    """
    List all credentials in the current wallet.
    """

    # for now, we have our secret password in our wallet config
    wallet_name = wallet.wallet_name
    wallet_config = json.loads(wallet.wallet_config)
    raw_password = wallet_config['wallet_key']
    wallet_handle = open_wallet(wallet_name, raw_password)

    (search_handle, search_count) = run_coroutine_with_args(prover_search_credentials, wallet_handle, "{}")
    credentials = run_coroutine_with_args(prover_fetch_credentials, search_handle, search_count)
    run_coroutine_with_args(prover_close_credentials_search, search_handle)

    close_wallet(wallet_handle)

    return json.loads(credentials)


######################################################################
# basic wallet storage utilities (non-secrets interface)
######################################################################





from django.contrib import admin

from .models import *


admin.site.register(IndyUser)
admin.site.register(IndyOrgRole)
admin.site.register(IndyOrganization)
admin.site.register(IndyOrgRelationship)
admin.site.register(IndyWallet)
admin.site.register(IndySchema)
admin.site.register(IndyCredentialDefinition)
admin.site.register(IndyProofRequest)
admin.site.register(AgentConnection)
admin.site.register(AgentConversation)




import asyncio
import json
from os import environ
import random

from django.conf import settings

from indy import anoncreds, crypto, did, ledger, pool, wallet
from indy.error import ErrorCode, IndyError

from .utils import *
from .indy_state import get_pool_handle


######################################################################
# basic ledger query utilities
######################################################################
def get_did_info(my_did):
    """
    Lookup DID information on the ledger
    """
    pool_handle = get_pool_handle()

    get_nym_request = run_coroutine_with_args(ledger.build_get_nym_request, my_did, my_did)
    nym_response = run_coroutine_with_args(ledger.submit_request, pool_handle, get_nym_request)
    seq_no = int(json.loads(nym_response)['result']['seqNo'])
    get_txn_request = run_coroutine_with_args(ledger.build_get_txn_request, my_did, "DOMAIN", seq_no)
    txn_response = run_coroutine_with_args(ledger.submit_request, pool_handle, get_nym_request)

    return (nym_response, txn_response)

def get_did_attrib(my_did, attrib):
    """
    Lookup DID attributes on the ledger
    """
    pool_handle = get_pool_handle()

    get_attrib_request = run_coroutine_with_args(ledger.build_get_attrib_request, my_did, my_did, attrib, None, None)
    attrib_response = run_coroutine_with_args(ledger.submit_request, pool_handle, get_attrib_request)

    return attrib_response

def write_new_did(wallet_handle, ledger_did, my_did, verkey, alias, role):
    """
    Write a new DID to the ledger
    """
    pool_handle = get_pool_handle()

    req_json = run_coroutine_with_args(ledger.build_nym_request, ledger_did, my_did, verkey, alias, role)
    rv_json = run_coroutine_with_args(ledger.sign_and_submit_request, pool_handle, wallet_handle, ledger_did, req_json)

def write_did_attrib(wallet_handle, ledger_did, my_did, attrib_raw):
    """
    Add or update a DID attribute on the ledger
    """
    pool_handle = get_pool_handle()

    attrib_request = run_coroutine_with_args(ledger.build_attrib_request, ledger_did, my_did, None, attrib_raw, None)
    run_coroutine_with_args(ledger.sign_and_submit_request, pool_handle, wallet_handle, my_did, attrib_request)

def get_schema_info(my_did, schema_id):
    """
    Read info from the ledger
    """
    pool_handle = get_pool_handle()

    get_schema_request = run_coroutine_with_args(ledger.build_get_schema_request, my_did, schema_id)
    get_schema_response = run_coroutine_with_args(ledger.submit_request, pool_handle, get_schema_request)

    return get_schema_response

def get_cred_def_info(my_did, cred_def_id):
    """
    Read info from the ledger
    """
    pool_handle = get_pool_handle()

    get_cred_def_request = run_coroutine_with_args(ledger.build_get_cred_def_request, my_did, cred_def_id)
    get_cred_def_response = run_coroutine_with_args(ledger.submit_request, pool_handle, get_cred_def_request)

    return get_cred_def_response





from django.urls import path, include
from django.conf.urls import url
from django.contrib.auth import views as auth_views
from django.conf import settings

#from .forms import *
from .views import *


#app_name = "indy"

urlpatterns = [
    path('accounts/', include('django.contrib.auth.urls')),
    path('signup/', user_signup_view, name='signup'),
    path('org_signup/', org_signup_view, name='org_signup'),
    path('mobile_request/', mobile_request_connection, name='mobile_request'),
    path('send_invitation/', handle_connection_request, name='send_invitation'),
    path('list_connections/', list_connections, name='list_connections'),
    path('connection_response/', handle_connection_response, name='connection_response'),
    path('check_connection/', poll_connection_status, name='check_connection'),
    path('invitation/<token>', connection_qr_code, name='connection_qr'),
    path('form_response/', form_response, name='form_response'),
    path('check_messages/', check_connection_messages, name='check_messages'),
    path('list_conversations/', list_conversations, name='list_conversations'),
    path('cred_offer_response/', handle_cred_offer_response, name='cred_offer_response'),
    path('proof_req_response/', handle_proof_req_response, name='proof_req_response'),
    path('proof_select_claims/', handle_proof_select_claims, name='proof_select_claims'),
    path('select_credential_offer/', handle_select_credential_offer, name='select_credential_offer'),
    path('credential_offer/', handle_credential_offer, name='credential_offer'),
    path('select_proof_request/', handle_select_proof_request, name='select_proof_request'),
    path('send_proof_request/', handle_send_proof_request, name='send_proof_request'),
    path('view_proof/', handle_view_proof, name='view_proof'),
    path('check_conversation/', poll_conversation_status, name='check_conversation'),
    path('list_credentials/', list_wallet_credentials, name='list_credentials'),
    path('profile/', plugin_view, name='indy_profile', kwargs={'view_name': 'INDY_PROFILE_VIEW'}),
    path('data/', plugin_view, name='indy_data', kwargs={'view_name': 'INDY_DATA_VIEW'}),
    path('wallet/', plugin_view, name='indy_wallet', kwargs={'view_name': 'INDY_WALLET_VIEW'}),
    path('connections/', list_connections, name='connections'),
    path('conversations/', list_conversations, name='conversations'),
    path('credentials/', list_wallet_credentials, name='credentials'),
    path('', auth_views.LoginView.as_view(), name='login'),
]

